@@ -1,5 +1,6 @@
 from pyspark.sql import DataFrame
 from pyspark.ml.feature import VectorAssembler, PCA as SparkPCA
+from ..testing import assert_has_columns
 from .Scaler import Scaler
 from .disassemble import disassemble
 
@@ -50,6 +51,8 @@ class PCA:
 		"""
 		if self._columns is None:
 			self._columns = X.columns
+		else:
+			assert_has_columns(X, columns=self._columns)
 
 		if self._scale:
 			self._scaler = Scaler(columns=self._columns, with_mean=self._scale_with_means, with_sd=self._scale_with_sd)
@@ -74,6 +77,8 @@ class PCA:
 		if not self._fitted:
 			self.fit(X=X)
 
+		assert_has_columns(X, self._columns)
+
 		if self._scale:
 			scaled = self._scaler.transform(X=X, return_vector=True)
 		else:
