@@ -1,7 +1,6 @@
 from pandas import DataFrame as PandasDF
 from pyspark.sql import DataFrame as SparkDF
 from pyspark.sql import functions as f
-from pyspark.sql.types import IntegerType, FloatType, LongType, DateType, TimestampType, BooleanType
 from ...functions import get_function_arguments
 
 
@@ -10,7 +9,7 @@ def apply_function_in_a_simple_way(data, function, new_column, return_type=None)
 	:type data: PandasDF or SparkDF
 	:type function: callable
 	:type new_column: str
-	:type return_type: IntegerType, FloatType, LongType, DateType, TimestampType, BooleanType
+	:type return_type:
 	"""
 	args = get_function_arguments(function=function)
 	missing_columns = [arg for arg in args if arg not in data.columns]
@@ -22,12 +21,14 @@ def apply_function_in_a_simple_way(data, function, new_column, return_type=None)
 			raise ValueError(f'return_type should be provided for pyspark DataFrame.')
 		data = data.withColumn(
 			new_column,
-			f.udf(function, return_type)(*args)
+			f.udf(function, return_type)(*[f.col(arg) for arg in args])
 		)
 
 	elif isinstance(data, PandasDF):
 		data = data.copy()
-		data[new_column] = data.apply(lambda row: function(row[key] for key in args), axis=1)
+		def _function(row):
+			return function(*[row[arg] for arg in args])
+		data[new_column] = data.apply(lambda row: _function(row), axis=1)
 
 	else:
 		raise TypeError(f'data of type "{type(data)}" is not supported!')
@@ -35,9 +36,6 @@ def apply_function_in_a_simple_way(data, function, new_column, return_type=None)
 	return data
 
 
-
-
-
 def apply_function(data, function, new_column, return_type=None, unique_ratio=0.01, echo=0):
 	"""
 	takes a function and applies it to all rows of the data
@@ -70,8 +68,8 @@ def apply_function(data, function, new_column, return_type=None, unique_ratio=0.
 
 		if isinstance(data, PandasDF):
 			uniques = data[args].drop_duplicates()
-			n_uniques = uniques.size[0]
-			n_total = data.size[0]
+			n_uniques = uniques.shape[0]
+			n_total = data.shape[0]
 		elif isinstance(data, SparkDF):
 			uniques = data.select(*args).distinct()
 			n_uniques = uniques.count()
@@ -88,9 +86,13 @@ def apply_function(data, function, new_column, return_type=None, unique_ratio=0.
 			if echo:
 				print(f'applying the function only on {n_uniques} unique cases instead of {n_total} cases.')
 
-			uniques = apply_function_in_a_simple_way(data=uniques, function=function, new_column=new_column)
+			uniques = apply_function_in_a_simple_way(
+				data=uniques, function=function, return_type=return_type, new_column=new_column
+			)
+
+			columns = list(data.columns) + [new_column]
 
 			if isinstance(data, PandasDF):
-				return uniques.merge(data, on=args, how='left')
+				return uniques.merge(data, on=args, how='left')[columns]
 			else:
-				return uniques.join(data, on=args, how='left')
+				return uniques.join(data, on=args, how='left').select(*columns)
